/**
 * The function for rendering kanban item
 */
function renderKanbanItem(item, $item)
{
    var $title = $item.children('.title');
    if(!$title.length)
    {
        $title = $('<a class="title" data-toggle="modal" data-width="80%"></a>')
            .appendTo($item);
    }
    $title.text(item.name);
    $title.attr('href', createLink('sys.task', 'view', 'taskID=' + item.id + '&from=kanban'));

    var $info = $item.children('.info');
    if(!$info.length) $info = $(
    [
        '<div class="info">',
            '<span class="pri"></span>',
            '<span class="time label label-light"></span>',
            '<div class="user"></div>',
        '</div>'
    ].join('')).appendTo($item);

    $item.data('task', item);

    $info.children('.pri')
        .attr('class', 'pri label-pri label-pri-' + item.pri)
        .text(item.pri);

    var $time = $info.children('.time');
    if(item.deadline && item.deadline !== '0000-00-00')
    {
        var deadline = $.zui.createDate(item.deadline);
        var today    = $.zui.createDate(v.today);
        var isExpired = deadline.getTime() < today.getTime();
        var dateFormat = (today.getFullYear() === deadline.getFullYear() ? 'MM-dd ' : 'yyyy-MM-dd ') + v.kanbanLang.deadline;
        $time.text($.zui.formatDate(deadline, dateFormat))
            .toggleClass('text-red', isExpired)
            .show();
    }
    else
    {
        $time.hide();
    }

    var $user = $info.children('.user');
    if(item.assignedTo)
    {
        var member = v.members[item.assignedTo] || {account: item.assignedTo};
        var canAssign = item.status !== 'closed' && item.status !== 'cancel';
        var $avatar = $(
            canAssign ? '<a class="avatar avatar-sm avatar-user" data-toggle="modal"></a>'
                : '<div class="avatar avatar-sm avatar-user"></div>'
            ).attr('data-name', member.realname || member.account)
            .attr('data-id', member.id || member.account);
        if(member.avatar)
        {
            $avatar.append('<img src="' + member.avatar + '" />');
        }
        $avatar.renderAvatar();
        if(canAssign)
        {
            $avatar.attr('href', createLink('sys.task', 'assignto', 'taskID=' + item.id + '&fromKanban=1'));
        }

        $user.empty()
            .append($avatar)
            .attr('title', member.realname || member.account)
            .show();
    }
    else
    {
        $user.hide();
    }
}

/**
 * Show error message
 * @param {string|object} message Message
 */
function showErrorMessager(message)
{
    var html = false;
    if(message instanceof Error)
    {
        message = message.message;
    }
    else if(typeof message === 'object')
    {
        html = [];
        $.each(message, function(key, msg)
        {
            html.push($.isArray(msg) ? msg.join('') : String(msg));
        });
        message = html.join('<br/>');
    }
    else
    {
        message = String(message);
    }

    if(typeof message === 'string' && message.length)
    {
        $.zui.messager.danger(message, {html: !!html});
    }
}

/**
 * Update kanban
 * @param {string} [kanbanID] Kanban ID
 * @param {function(Error)} [callback] Callback on completed
 */
function updateKanban(kanbanID, callback)
{
    if(typeof kanbanID == 'function')
    {
        callback = kanbanID;
        kanbanID = 0;
    }

    var url = createLink('kanban', 'ajaxGetData', 'projectID=' + v.projectID + '&kanbanID=' + (kanbanID || 0));
    $.ajax(
    {
        method:   'get',
        dataType: 'json',
        url:      url,
        success: function(data)
        {
            var kanban = $('#kanban').data('zui.kanban');
            if(kanbanID) kanban.renderKanban(data);
            else kanban.render(data);
            typeof callback === 'function' && callback();
        },
        error: function(xhr, status, error)
        {
            showErrorMessager(error || v.lang.timeout);
            typeof callback === 'function' && callback(error);
        }
    });
}

/**
 * Hide kanban action
 */
function hideKanbanAction()
{
    $('#kanban').attr('data-action-enabled', null);
}

/**
 * Create kanban column
 * @param {string} name Column name
 * @param {string} kanbanID Kanban ID
 * @param {function(Error)} callback Callback on complted
 */
function createKanbanColumn(name, kanbanID, callback)
{
    var url = createLink('kanban', 'createColumn', 'kanbanID=' + kanbanID);
    var postData = typeof name === 'object' ? name : {name: name};
    $.ajax(
    {
        method:   'post',
        dataType: 'json',
        url:      url,
        data:     postData,
        success: function(data)
        {
            if(data && data.result === 'success')
            {
                updateKanban(kanbanID, callback);
            }
            else
            {
                callback && callback(data);
            }
        },
        error: function(xhr, status, error)
        {
            callback && callback(error);
        }
    });
}

/**
 * Open add column form
 * @param {JQuery} $element Trigger element
 */
function openAddColForm($element)
{
    var $col  = $element.closest('.kanban-col');
    var $form = $('#kanbanAddColForm').appendTo($col);

    if(!$form.data('inited'))
    {
        $form.data('inited', true);
        $form.on('click', '.action-submit', function()
        {
            var $input = $form.find('input');
            var name = $input.val();
            if(!name.length)
            {
                $input.focus().parent().addClass('has-error');
                return;
            }

            $form.addClass('loading');
            var kanbanID = $form.closest('.kanban-board').data('id');
            createKanbanColumn(name, kanbanID, function(error)
            {
                $form.removeClass('loading');
                if(error)
                {
                    $input.focus().parent().addClass('has-error');
                }
                else
                {
                    hideKanbanAction();
                }
            });
        });
        $form.on('change input', function()
        {
            $form.find('.has-error').removeClass('has-error');
        });
    }

    $form.find('input').val('').focus();
}

/**
 * Open header more contextmenu
 * @param {JQuery} $element Trigger element
 */
function openColMoreMenu($element)
{
    var $menu = $('#kanbanColActions');
    var statusType = $element.closest('.kanban-col').attr('data-type');
    $element.parent()
        .addClass('btn-group pull-right')
        .append($menu);
    $menu.find('.action')
        .removeClass('active')
        .filter('[data-status="' + statusType + '"]')
        .addClass('active');
}

/**
 * Delete column
 * @param {JQuery} $element Trigger element
 */
function deleteKanbanCol($element)
{
    if(!confirm(v.lang.confirmDelete)) return;

    var columnID = $element.closest('.kanban-col').data('id');
    var kanbanID = $element.closest('.kanban-board').data('id');
    $('#kanbanColActions').appendTo('body');
    $.ajax(
    {
        method: 'get',
        url: createLink('kanban', 'deleteColumn', 'columnID=' + columnID),
        dataType: 'json',
        success: function(data)
        {
            if(data && data.result === 'success')  updateKanban(kanbanID);
            else showErrorMessager(data && data.message);
        },
        error: function(xhr, status, error)
        {
            showErrorMessager(error || v.lang.timeout);
        }
    });
}

/**
 * Change column status
 * @param {JQuery} $element Trigger element
 */
function changeColStatus($element)
{
    var newStatus = $element.data('status');
    var columnID  = $element.closest('.kanban-col').data('id');
    var kanbanID  = $element.closest('.kanban-board').data('id');
    var url = createLink('kanban', 'editColumn', 'columnID=' + columnID);
    $.ajax(
    {
        method:   'post',
        dataType: 'json',
        url:      url,
        data:     {taskStatus: newStatus},
        success: function(data)
        {
            if(data && data.result === 'success')  updateKanban(kanbanID);
            else showErrorMessager(data && data.message);
        },
        error: function(xhr, status, error)
        {
            showErrorMessager(error || v.lang.timeout);
        }
    });
}

/**
 * Add task to kanban
 * @param {Object} task The new task object
 * @param {string} kanbanID Kanban ID
 * @param {function(Error)} callback Callback on complted
 */
function addTaskToKanban(task, kanbanID, callback)
{
    var url = createLink('task', 'create', 'projectID=' + v.projectID);
    $.ajax(
    {
        method:   'post',
        dataType: 'json',
        url:      url,
        data:     task,
        success: function(data)
        {
            if(data && data.result === 'success')
            {
                updateKanban(kanbanID, callback);
            }
            else
            {
                callback && callback(data.message || data);
            }
        },
        error: function(xhr, status, error)
        {
            callback && callback(error || v.lang.timeout);
        }
    });
}

/**
 * Open form for adding task
 * @param {JQuery} $element Trigger element
 */
function openAddTaskForm($element)
{
    var $col  = $element.closest('.kanban-col');
    var $form = $('#kanbanAddTaskForm').appendTo($col);

    if(!$form.data('inited'))
    {
        $form.data('inited', true);
        $form.on('click', '.action-submit', function()
        {
            var $name = $form.find('input[name="name"]');
            var name = $name.val();
            if(!name.length)
            {
                $name.focus().parent().addClass('has-error');
                return;
            }

            $form.addClass('loading');

            var kanbanID   = $form.closest('.kanban-board').data('id');
            var statusType = $form.closest('.kanban-col').data('type');
            var laneID     = $form.closest('.kanban-lane').data('id');

            var task =
            {
                name:       name,
                desc:       $form.find('[name="desc"]').val(),
                pri:        $form.find('[name="pri"]').val(),
                assignedTo: $form.find('[name="assignedTo"]').val(),
                estimate:   $form.find('[name="estimate"]').val(),
                deadline:   $form.find('[name="deadline"]').val(),
                status:     statusType,
                kanbanLane: laneID
            };
            addTaskToKanban(task, kanbanID, function(error)
            {
                $form.removeClass('loading');
                if(error)
                {
                    showErrorMessager(error);
                }
                else
                {
                    hideKanbanAction();
                }
            });
        });
        $form.on('change input', function(e)
        {
            $(e.target).closest('.has-error').removeClass('has-error');
        });
    }

    $form.find('[name="desc"]').val('');
    $form.find('[name="pri"]').val(3);
    $form.find('[name="assignedTo"]').val(v.account);
    $form.find('[name="estimate"]').val('');
    $form.find('[name="deadline"]').val('');
    $form.find('[name="name"]').val('').focus();
}

/**
 * Edit lane name
 * @param {string} name Lane name
 * @param {string} laneID Lane ID
 * @param {string} kanbanID Kanban ID
 * @param {function(Error)} callback Callback on complted
 */
function editLaneName(name, laneID, kanbanID, callback)
{
    var url = createLink('kanban', 'editLane', 'laneID=' + laneID);
    $.ajax(
    {
        method:   'post',
        dataType: 'json',
        url:      url,
        data:     {name: name},
        success: function(data)
        {
            if(data && data.result === 'success')
            {
                updateKanban(kanbanID, callback);
            }
            else
            {
                callback && callback(data);
            }
        },
        error: function(xhr, status, error)
        {
            callback && callback(error);
        }
    });
}

/**
 * Open form for editing lane name
 * @param {JQuery} $element Trigger element
 */
function openEditLaneForm($element)
{
    var $lane = $element.closest('.kanban-lane');
    var $form = $('#kanbanEditLaneForm').appendTo($lane);

    if(!$form.data('inited'))
    {
        $form.data('inited', true);
        $form.on('click', '.action-submit', function()
        {
            var $input = $form.find('input');
            var name   = $input.val();
            var $lane  = $form.closest('.kanban-lane');
            if(!name.length)
            {
                $input.focus().parent().addClass('has-error');
                return;
            }

            if(name === $lane.data('lane')['name'])
            {
                hideKanbanAction();
                return;
            }

            $form.addClass('loading');
            var laneID   = $lane.data('id');
            var kanbanID = $lane.closest('.kanban-board').data('id');
            editLaneName(name, laneID, kanbanID, function(error)
            {
                $form.removeClass('loading');
                if(error)
                {
                    $input.focus().parent().addClass('has-error');
                }
                else
                {
                    hideKanbanAction();
                }
            });
        });
        $form.on('change input', function()
        {
            $form.find('.has-error').removeClass('has-error');
        });
    }

    var lane = $lane.data('lane');
    $form.find('input').val(lane.name).focus();
}

/**
 * Edit lane name
 * @param {string} name Column name
 * @param {string} columnID Column ID
 * @param {string} kanbanID Kanban ID
 * @param {function(Error)} callback Callback on complted
 */
 function editColumnName(name, columnID, kanbanID, callback)
 {
     var url = createLink('kanban', 'editColumn', 'columnID=' + columnID);
     $.ajax(
     {
         method:   'post',
         dataType: 'json',
         url:      url,
         data:     {name: name},
         success: function(data)
         {
             if(data && data.result === 'success')
             {
                 updateKanban(kanbanID, callback);
             }
             else
             {
                 callback && callback(data);
             }
         },
         error: function(xhr, status, error)
         {
             callback && callback(error);
         }
     });
 }

/**
 * Open form for editing column name
 * @param {JQuery} $element Trigger element
 */
function openEditColForm($element)
{
    var $form = $('#kanbanEditColForm');
    if(!$form.data('inited'))
    {
        $form.data('inited', true);
        $form.on('click', '.action-submit', function()
        {
            var $input = $form.find('input');
            var name   = $input.val();
            if(!name.length)
            {
                $input.focus().parent().addClass('has-error');
                return;
            }

            var $col = $form.closest('.kanban-col');
            if(name === $col.data('col')['name'])
            {
                hideKanbanAction();
                return;
            }

            $form.addClass('loading');
            var kanbanID = $col.closest('.kanban-board').data('id');
            var columnID = $col.data('id');
            editColumnName(name, columnID, kanbanID, function(error)
            {
                $form.removeClass('loading');
                if(error)
                {
                    $input.focus().parent().addClass('has-error');
                }
                else
                {
                    hideKanbanAction();
                }
            });
        });
        $form.on('change input', function()
        {
            $form.find('.has-error').removeClass('has-error');
        });
    }

    var $col = $element.closest('.kanban-col');
    $form.appendTo($col).find('.input-group').fixInputGroup();
    $form.find('input').val($col.data('col')['name']).focus();
}

/**
 * Reset lane height according to window height.
 */
function resetLaneHeight()
{
    var maxHeight = '360px';
    if(v.laneCount < 2)
    {
        var windowHeight = $(window).height();
        var marginTop    = $('#mainContent').css('margin-top');
        var headerHeight = $('#kanban > .kanban-board:first > .kanban-header').outerHeight();
        var actionHeight = $('#kanban > .kanban-board:first > .kanban-lane > .kanban-col:first > .kanban-lane-actions').outerHeight();

        maxHeight = windowHeight - parseInt(marginTop) - headerHeight - actionHeight;
    }
    $('.kanban-lane-items').css('max-height', maxHeight);
}

/**
 * Close modal and update kanban data.
 */
function closeModalAndUpdateKanban()
{
    updateKanban();
    $.zui.closeModal();
}

/**
 * Handle create lane.
 */
function handleCreateLane()
{
    closeModalAndUpdateKanban();
    resetLaneHeight();
}

/**
 * Status change map
 */
var statusChangeMap = {
    wait:   ['doing', 'done', 'cancel'],
    doing:  ['done', 'cancel'],
    done:   ['doing', 'closed'],
    cancel: ['doing', 'closed'],
    closed: ['doing']
};

/**
 * Find drop columns
 * @param {JQuery} $element Drag element
 * @param {JQuery} $root Dnd root element
 */
function findDropColumns($element, $root)
{
    var $task  = $element;
    var task   = $task.data('task');
    var status = task.status;
    var $col   = $task.closest('.kanban-col');
    var col    = $col.data();
    var lane   = $col.closest('.kanban-lane').data('lane');
    var allStatusCanChange = statusChangeMap[status];

    return $root.find('.kanban-lane-col:not([data-type="EMPTY"],[data-type=""])').filter(function()
    {
        var $newCol = $(this);
        var newCol = $newCol.data();
        var $newLane = $newCol.closest('.kanban-lane');
        var newLane = $newLane.data('lane');
        var canChangeCol = allStatusCanChange.indexOf(newCol.type) > -1;
        var canChangeLane = newLane.id !== lane.id && col.type === newCol.type;
        if(canChangeCol || canChangeLane)
        {
            $newCol.addClass('can-drop-here');
            return true;
        }
    });
}

/**
 * Handle drop task
 * @param {Object} event Drop event object
 */
function handleDropTask($element, event, kanban)
{
    if(!event.target || !event.isNew) return;

    var $task   = $element;
    var $oldCol = $task.closest('.kanban-col');
    var $newCol = $(event.target).closest('.kanban-col');
    var oldCol  = $oldCol.data();
    var newCol  = $newCol.data();
    var oldLane = $oldCol.closest('.kanban-lane').data('lane');
    var newLane = $newCol.closest('.kanban-lane').data('lane');

    if(oldCol.id === newCol.id && newLane.id === oldLane.id) return false;

    var newStatus = newCol.type;
    var task = $task.data('task');

    /* Task status not change */
    if(newStatus === task.status && task.kanbanLane !== newLane.id)
    {
        var url = createLink('sys.task', 'edit', 'taskID=' + task.id);
        return $.ajax(
        {
            method:   'post',
            dataType: 'json',
            url:      url,
            data:     {changeLane: true, kanbanLane: newLane.id},
            success: function(data)
            {
                if(data && data.result === 'success')  updateKanban();
                else showErrorMessager(data && data.message);
            },
            error: function(xhr, status, error)
            {
                showErrorMessager(error || v.lang.timeout);
            }
        });
    }

    /* Show dialog to user for changing status */
    var methodToChangeStatus =
    {
        wait:   'activate',
        doing:  'start',
        done:   'finish',
        cancel: 'cancel',
        closed: 'close',
    };
    var modalUrl = createLink('sys.task', methodToChangeStatus[newStatus], 'taskID=' + task.id + '&laneID=' + newLane.id);
    $.zui.modalTrigger.show(
    {
        url: modalUrl,
        onHide: updateKanban.bind(this, task.kanbanLane !== newLane.id ? 0 : $newCol.closest('.kanban-board').data('id'))
    });
}

/**
 * Handle finish drop task
 */
function handleFinishDrop()
{
    $('#kanban').find('.can-drop-here').removeClass('can-drop-here');
}

/**
 * Adjust add button postion in column
 */
function adjustAddBtnPosition($kanban)
{
    if(!$kanban)
    {
        $('#kanban').children('.kanban-board').each(function()
        {
            adjustAddBtnPosition($(this));
        });
        return;
    }

    $kanban.find('.kanban-lane-col:not([data-type="EMPTY"])').each(function()
    {
        var $col = $(this);
        var items = $col.children('.kanban-lane-items')[0];
        $col.toggleClass('has-scrollbar', items.scrollHeight > items.clientHeight);
    });
}

/**
 * Kanban action handlers
 */
var kanbanActionHandlers =
{
    addCol:          openAddColForm,
    headerMore:      openColMoreMenu,
    deleteCol:       deleteKanbanCol,
    changeColStatus: changeColStatus,
    addItem:         openAddTaskForm,
    editLaneName:    openEditLaneForm,
    editCol:         openEditColForm,
    dropItem:        handleDropTask
};

/**
 * Handle kanban action
 */
function handleKanbanAction(action, $element, event, kanban)
{
    $('#kanban').attr('data-action-enabled', action);
    var handler = kanbanActionHandlers[action];
    if(handler) handler($element, event, kanban);
}

/**
 * Init when page ready
 */
$(function()
{
    /* Init kanban */
    var $kanban = $('#kanban').kanban(
    {
        data:             v.kanbanList,
        minColumnWidth:   250,
        maxColHeight:     510,
        createColumnText: v.kanbanLang.createColumn,
        addItemText:      v.kanbanLang.addItem,
        itemRender:       renderKanbanItem,
        onAction:         handleKanbanAction,
        onRenderKanban:   adjustAddBtnPosition,
        droppable:
        {
            target:       findDropColumns,
            finish:       handleFinishDrop,
            mouseButton: 'left'
        }
    }).on('click', '.action-cancel', hideKanbanAction);

    /* Hide action box when user click document */
    $(document).on('click', function(e)
    {
        var currentAction = $kanban.attr('data-action-enabled');
        var canHideAction = (currentAction === 'headerMore' || currentAction === 'editLaneName')
            && !$(e.target).closest('.action,.action-box').length;
        if(canHideAction) hideKanbanAction();
    });

    /* Adjust the add button position on window resize */
    $(window).on('resize', function(){adjustAddBtnPosition();});

    resetLaneHeight();
});
